/* ISC license. */

#include <string.h>
#include <sys/stat.h>

#include <skalibs/posixplz.h>
#include <skalibs/strerr2.h>
#include <skalibs/sgetopt.h>
#include <skalibs/stralloc.h>
#include <skalibs/djbunix.h>

#define USAGE "s6-instance-create template rwdir value"
#define dieusage() strerr_dieusage(100, USAGE)
#define dienomem() strerr_diefu1sys(111, "stralloc_catb")
#define dienosvd() strerr_dief1x(100, argv[0], " is not a valid service directory")

int main (int argc, char const *const *argv)
{
  stralloc sa = STRALLOC_ZERO ;
  PROG = "s6-instance-create" ;
  {
    subgetopt_t l = SUBGETOPT_ZERO ;
    for (;;)
    {
      int opt = subgetopt_r(argc, argv, "", &l) ;
      if (opt == -1) break ;
      switch (opt)
      {
        default : dieusage() ;
      }
    }
    argc -= l.ind ; argv += l.ind ;
  }
  if (argc < 3) dieusage() ;
  if (strchr(argv[2], '\n') || strchr(argv[2], '/'))
    strerr_dief1x(100, " instance values cannot contain slashes or newlines") ;

  {
    struct stat st ;
    if (stat(argv[1], &st) < 0)
      strerr_diefu2sys(111, "stat ", argv[0]) ;
    if (!S_ISDIR(st.st_mode)) dienosvd() ;
    if (!stralloc_cats(&sa, argv[1])
     || !stralloc_catb(&sa, "/run", 5)) dienomem() ;
    if (stat(sa.s, &st) < 0)
      strerr_diefu2sys(111, "stat ", sa.s) ;
    if (!(st.st_mode & S_IXUSR)) dienosvd() ;
  }
  sa.len = 0 ;
  if (!stralloc_cats(&sa, argv[1])
   || !stralloc_catb(&sa, "/", 1)
   || !sabasename(&sa, argv[0], strlen(argv[1]))
   || !stralloc_catb(&sa, "@", 1)
   || !stralloc_cats(&sa, argv[2])
   || !stralloc_0(&sa)) dienomem() ;
  if (!hiercopy(argv[1], sa.s))
    strerr_diefu4sys(111, "recursively copy ", argv[0], " to ", sa.s) ;
  sa.len-- ;
  if (!stralloc_catb(&sa, "/down", 6)) dienomem() ;
  if (down)
  {
    int fd = open_create(sa.s) ;
    if (fd < 0) strerr_diefu2sys(111, "create ", sa.s) ;
    close(fd) ;
  }
  else unlink_void(sa.s) ;
  return 0 ;
}
